/*     */ package com.netapp.asup.cache.LRU;
/*     */ 
/*     */ import com.googlecode.concurrentlinkedhashmap.ConcurrentLinkedHashMap;
/*     */ import com.googlecode.concurrentlinkedhashmap.ConcurrentLinkedHashMap.Builder;
/*     */ import com.googlecode.concurrentlinkedhashmap.EvictionListener;
/*     */ import com.googlecode.concurrentlinkedhashmap.Weigher;
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asup.config.Configuration;
/*     */ import com.netapp.asup.util.AsupCacheConstants;
/*     */ import java.util.HashSet;
/*     */ import java.util.Map;
/*     */ import java.util.Set;
/*     */ import java.util.concurrent.Executors;
/*     */ import java.util.concurrent.ScheduledExecutorService;
/*     */ import java.util.concurrent.ThreadFactory;
/*     */ import java.util.concurrent.TimeUnit;
/*     */ 
/*     */ 
/*     */ public class AsupLRUCacheImpl<K, V>
/*     */   implements AsupLRUCache<K, V>
/*     */ {
/*  22 */   private static final AsupLogger LOGGER = AsupLogger.getLogger(AsupLRUCacheImpl.class);
/*     */   
/*     */ 
/*     */   private ConcurrentLinkedHashMap<K, V> cache;
/*     */   
/*  27 */   private static AsupLRUCache<String, Object> instance = null;
/*     */   
/*  29 */   private WeigherFactory factory = WeigherFactory.getInstance();
/*     */   
/*  31 */   private int cacheSize = 0;
/*     */   
/*  33 */   private int maxThresholdSize = 0;
/*     */   
/*  35 */   private int afterEvictionThresholdSize = 0;
/*     */   
/*  37 */   private WeightType weightType = WeightType.ENTRY;
/*     */   
/*     */ 
/*  40 */   private final AsupLRUCacheStats stats = new AsupLRUCacheStats();
/*     */   
/*     */ 
/*  43 */   private final ScheduledExecutorService statisticsThreadPool = Executors.newScheduledThreadPool(1, new StatisticsThreadFactory());
/*     */   
/*     */ 
/*     */ 
/*  47 */   private final ScheduledExecutorService evictionThreadPool = Executors.newScheduledThreadPool(1, new EvictionThreadFactory());
/*     */   
/*     */ 
/*     */   private static final String STATISTICS_THREAD_NAME = "AsupCache.StatisticsThread";
/*     */   
/*     */   private static final String EVICTION_THREAD_NAME = "AsupCache.ThresholdEviction";
/*     */   
/*  54 */   private Set<EvictionListener<K, V>> registeredListeners = new HashSet();
/*     */   
/*     */   public static void initialize(Configuration config) {
/*  57 */     instance = new AsupLRUCacheImpl(config);
/*     */   }
/*     */   
/*     */   public static void initialize(String weightTypeValue, int cacheSize, double maxThresholdPercent, double afterEvictionThresholdPercent) {
/*  61 */     instance = new AsupLRUCacheImpl(weightTypeValue, cacheSize, maxThresholdPercent, afterEvictionThresholdPercent);
/*     */   }
/*     */   
/*     */   public static AsupLRUCache<String, Object> getinstance() {
/*  65 */     return instance;
/*     */   }
/*     */   
/*     */   private AsupLRUCacheImpl(String weightTypeValue, int cacheSize, double maxThresholdPercent, double afterEvictionThresholdPercent) {
/*  69 */     this.weightType = WeightType.getWeightType(weightTypeValue);
/*  70 */     this.cacheSize = cacheSize;
/*  71 */     LOGGER.info("-9999", "The maximum size of cache defined in number of " + this.weightType.toString() + " :" + cacheSize + " ");
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*  76 */     ThresholdUnit afterEvictionThresholdUnit = new ThresholdUnit(cacheSize, afterEvictionThresholdPercent);
/*     */     
/*     */ 
/*  79 */     ThresholdUnit thresholdUnit = new ThresholdUnit(cacheSize, maxThresholdPercent);
/*     */     
/*     */ 
/*  82 */     int evictionThreadIntervalSeconds = 60;
/*     */     
/*  84 */     this.maxThresholdSize = thresholdUnit.unitsValue();
/*  85 */     this.afterEvictionThresholdSize = afterEvictionThresholdUnit.unitsValue();
/*     */     
/*  87 */     buildCache(thresholdUnit, afterEvictionThresholdUnit, evictionThreadIntervalSeconds);
/*     */   }
/*     */   
/*     */ 
/*     */   private void buildCache(ThresholdUnit thresholdUnit, ThresholdUnit afterEvictionThresholdUnit, int evictionThreadIntervalInSeconds)
/*     */   {
/*  93 */     buildAndInitializeCache();
/*  94 */     AsupLRUCacheImpl<K, V>.EagerlyEvictionPolicy evictionPolicy = new EagerlyEvictionPolicy(thresholdUnit, afterEvictionThresholdUnit);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 101 */     this.evictionThreadPool.scheduleAtFixedRate(evictionPolicy, 0L, evictionThreadIntervalInSeconds, TimeUnit.SECONDS);
/*     */     
/*     */ 
/*     */ 
/* 105 */     this.statisticsThreadPool.scheduleAtFixedRate(new StatisticsThread(this), 0L, 300L, TimeUnit.SECONDS);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private AsupLRUCacheImpl(Configuration config)
/*     */   {
/* 115 */     String weightTypeValue = config.getString(AsupCacheConstants.CACHE_WEIGHT_TYPE_KEY);
/*     */     
/* 117 */     this.weightType = WeightType.getWeightType(weightTypeValue);
/*     */     
/* 119 */     this.cacheSize = config.getInt(AsupCacheConstants.MAX_CACHE_SIZE_KEY, this.weightType.getDefaultMaxValue());
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 124 */     LOGGER.info("-9999", "The maximum size of cache defined in number of " + this.weightType.toString() + " :" + this.cacheSize + " ");
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 129 */     ThresholdUnit thresholdUnit = getThrsholdUnit(config);
/* 130 */     ThresholdUnit afterEvictionThresholdUnit = getAfterEvictionThresholdUnit(config);
/*     */     
/* 132 */     int evictionThreadIntervalInSeconds = config.getInt(AsupCacheConstants.EVICTION_THREAD_TIME_PERIOD, 60);
/*     */     
/*     */ 
/*     */ 
/* 136 */     this.maxThresholdSize = thresholdUnit.unitsValue();
/* 137 */     this.afterEvictionThresholdSize = afterEvictionThresholdUnit.unitsValue();
/*     */     
/* 139 */     LOGGER.info("-9999", getSizeMessage());
/*     */     
/* 141 */     buildCache(thresholdUnit, afterEvictionThresholdUnit, evictionThreadIntervalInSeconds);
/*     */   }
/*     */   
/*     */ 
/*     */   private String getSizeMessage()
/*     */   {
/* 147 */     StringBuilder builder = new StringBuilder();
/* 148 */     builder.append("The threshold size of cache defined in number of ");
/* 149 */     builder.append(this.weightType.toString());
/* 150 */     builder.append(" is:");
/* 151 */     builder.append(this.maxThresholdSize);
/* 152 */     builder.append(".The after eviction size of the cache defined in number of ");
/* 153 */     builder.append(this.weightType.toString());
/* 154 */     builder.append(" is:");
/* 155 */     builder.append(this.afterEvictionThresholdSize);
/* 156 */     builder.append('.');
/* 157 */     return builder.toString();
/*     */   }
/*     */   
/*     */   public V get(K key)
/*     */   {
/* 162 */     V data = this.cache.get(key);
/* 163 */     if (data == null) {
/* 164 */       this.stats.miss();
/* 165 */       return data;
/*     */     }
/* 167 */     this.stats.hit();
/* 168 */     return data;
/*     */   }
/*     */   
/*     */   public int size()
/*     */   {
/* 173 */     return this.cache.size();
/*     */   }
/*     */   
/*     */   public void clear()
/*     */   {
/* 178 */     this.cache.clear();
/*     */   }
/*     */   
/*     */   public V putIfAbsent(K key, V value)
/*     */   {
/* 183 */     this.stats.access();
/* 184 */     return (V)this.cache.putIfAbsent(key, value);
/*     */   }
/*     */   
/*     */   public V remove(K key)
/*     */   {
/* 189 */     V removedValue = this.cache.remove(key);
/* 190 */     if (null != removedValue) {
/* 191 */       this.stats.removed();
/*     */     }
/* 193 */     return removedValue;
/*     */   }
/*     */   
/*     */   public V put(K key, V value) {
/* 197 */     this.stats.access();
/* 198 */     return (V)this.cache.put(key, value);
/*     */   }
/*     */   
/*     */ 
/*     */   public AsupLRUCacheStats getStats()
/*     */   {
/* 204 */     return this.stats;
/*     */   }
/*     */   
/*     */   public String toString() {
/* 208 */     return this.cache.toString();
/*     */   }
/*     */   
/*     */ 
/*     */   public String getCacheStats()
/*     */   {
/* 214 */     long totalSize = this.cacheSize;
/* 215 */     StringBuilder statsMsg = new StringBuilder();
/* 216 */     statsMsg.append("LRU Stats: cacheTotalSizeIn" + this.weightType.toString());
/* 217 */     statsMsg.append("=");
/* 218 */     statsMsg.append(totalSize);
/* 219 */     statsMsg.append(", maxThresholdSize=");
/* 220 */     statsMsg.append(this.maxThresholdSize);
/* 221 */     statsMsg.append(", afterEvictionThresholdSize=");
/* 222 */     statsMsg.append(this.afterEvictionThresholdSize);
/* 223 */     statsMsg.append(", entries=");
/* 224 */     statsMsg.append(size());
/* 225 */     statsMsg.append("\n");
/* 226 */     statsMsg.append(this.stats.toString());
/* 227 */     statsMsg.append('.');
/*     */     
/* 229 */     return statsMsg.toString();
/*     */   }
/*     */   
/*     */   public void shutdown()
/*     */   {
/* 234 */     this.cache.clear();
/*     */     
/* 236 */     if (null != this.evictionThreadPool) {
/* 237 */       this.evictionThreadPool.shutdown();
/*     */     }
/* 239 */     if (null != this.statisticsThreadPool)
/* 240 */       this.statisticsThreadPool.shutdown();
/*     */   }
/*     */   
/*     */   private void logStats() {
/* 244 */     if (!LOGGER.isDebugEnabled())
/* 245 */       return;
/* 246 */     LOGGER.debug("-9999", getCacheStats());
/*     */   }
/*     */   
/*     */   private ThresholdUnit getAfterEvictionThresholdUnit(Configuration config)
/*     */   {
/* 251 */     double maximumAfterEvictionThreshold = config.getDouble(AsupCacheConstants.MAX_EVICTION_LIMIT_KEY, AsupCacheConstants.DEFAULT_AFTER_EVICTION_THRESHOLD);
/*     */     
/*     */ 
/* 254 */     ThresholdUnit afterEvictionThresholdUnit = new ThresholdUnit(this.cacheSize, maximumAfterEvictionThreshold);
/*     */     
/* 256 */     return afterEvictionThresholdUnit;
/*     */   }
/*     */   
/*     */   private ThresholdUnit getThrsholdUnit(Configuration config) {
/* 260 */     double minThresholdPercentage = config.getDouble(AsupCacheConstants.MAX_THRESHOLD_SIZE_KEY, AsupCacheConstants.DEFAULT_CACHE_THRESHOLD);
/*     */     
/*     */ 
/* 263 */     ThresholdUnit thresholdUnit = new ThresholdUnit(this.cacheSize, minThresholdPercentage);
/*     */     
/* 265 */     return thresholdUnit;
/*     */   }
/*     */   
/*     */   private void buildAndInitializeCache()
/*     */   {
/* 270 */     ConcurrentLinkedHashMap.Builder<K, V> maximumWeightedCapacity = new ConcurrentLinkedHashMap.Builder().maximumWeightedCapacity(this.cacheSize);
/*     */     
/*     */ 
/* 273 */     Weigher<?> weigher = this.factory.getWeigher(this.weightType);
/*     */     
/* 275 */     maximumWeightedCapacity.listener(new DefaultEvictionListener());
/*     */     
/* 277 */     this.registeredListeners.add(new AsupCacheEvictionListener());
/* 278 */     maximumWeightedCapacity.weigher(weigher);
/*     */     
/* 280 */     this.cache = maximumWeightedCapacity.build();
/*     */   }
/*     */   
/*     */   private void logEviction(K cacheKey, V value, boolean isThresholdEviction) {
/* 284 */     if (LOGGER.isTraceEnabled()) {
/* 285 */       LOGGER.trace("-9999", "The following cache entry is evicted from the cache, isThresholdEviction:" + isThresholdEviction + ",Key:" + cacheKey + ", Value:" + value);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   static class StatisticsThread
/*     */     extends Thread
/*     */   {
/*     */     AsupLRUCacheImpl<?, ?> lru;
/*     */     
/*     */ 
/*     */     public StatisticsThread(AsupLRUCacheImpl<?, ?> lru)
/*     */     {
/* 299 */       setDaemon(true);
/* 300 */       this.lru = lru;
/*     */     }
/*     */     
/*     */     public void run()
/*     */     {
/*     */       try {
/* 306 */         this.lru.logStats();
/*     */       }
/*     */       catch (Throwable e)
/*     */       {
/* 310 */         AsupLRUCacheImpl.LOGGER.warn("-9999", "Error occured during the asup cache statistics logging", e.getMessage());
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public class EagerlyEvictionPolicy
/*     */     extends Thread
/*     */   {
/*     */     private int maxThresholdValue;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     private int newShrinkedValue;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public EagerlyEvictionPolicy(AsupLRUCacheImpl.ThresholdUnit unit, AsupLRUCacheImpl.ThresholdUnit newShrinkUnit)
/*     */     {
/* 335 */       setDaemon(true);
/* 336 */       this.maxThresholdValue = unit.unitsValue();
/* 337 */       this.newShrinkedValue = newShrinkUnit.unitsValue();
/*     */     }
/*     */     
/*     */     public void run()
/*     */     {
/* 342 */       long startSize = AsupLRUCacheImpl.this.cache.weightedSize();
/* 343 */       int startNumberOfCacheEntries = AsupLRUCacheImpl.this.cache.size();
/* 344 */       long currentSize = startSize;
/* 345 */       int currentNumberOfCacheEntries = startNumberOfCacheEntries;
/* 346 */       int removedEntriesCount = 0;
/* 347 */       Map<K, V> ascendingMapWithLimit = null;
/* 348 */       V removedElement = null;
/*     */       
/*     */ 
/*     */ 
/*     */       try
/*     */       {
/* 354 */         while (currentSize > this.maxThresholdValue)
/*     */         {
/*     */ 
/*     */ 
/*     */ 
/* 359 */           long entriesToRemove = getNumberOfEntriesToBeRemoved(currentSize, currentNumberOfCacheEntries);
/*     */           
/* 361 */           int entriesToRemoveInt = (-2147483648L < entriesToRemove) && (2147483647L > entriesToRemove) ? (int)entriesToRemove : AsupLRUCacheImpl.this.cache.size();
/* 362 */           ascendingMapWithLimit = AsupLRUCacheImpl.this.cache.ascendingMapWithLimit(entriesToRemoveInt);
/*     */           
/*     */ 
/* 365 */           if (0 == ascendingMapWithLimit.size())
/*     */           {
/*     */ 
/* 368 */             AsupLRUCacheImpl.LOGGER.warn("-9999", "The size of the cache is above threshold, but not able to evict.");
/*     */             
/* 370 */             break;
/*     */           }
/*     */           
/* 373 */           for (K cacheKey : ascendingMapWithLimit.keySet()) {
/* 374 */             removedElement = AsupLRUCacheImpl.this.cache.remove(cacheKey);
/* 375 */             if (null != removedElement) {
/* 376 */               AsupLRUCacheImpl.this.logEviction(cacheKey, ascendingMapWithLimit.get(cacheKey), true);
/*     */               
/* 378 */               AsupLRUCacheImpl.this.stats.eagerEvict();
/* 379 */               removedEntriesCount++;
/*     */             }
/* 381 */             else if (AsupLRUCacheImpl.LOGGER.isDebugEnabled()) {
/* 382 */               AsupLRUCacheImpl.LOGGER.debug("-9999", "Eviction cannot remove the cache entry:" + cacheKey + ", Its already being removed from the cache.");
/*     */             }
/*     */           }
/*     */           
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 396 */           currentSize = AsupLRUCacheImpl.this.cache.weightedSize();
/* 397 */           currentNumberOfCacheEntries = AsupLRUCacheImpl.this.cache.size();
/*     */         }
/*     */         
/* 400 */         if (removedEntriesCount > 0) {
/* 401 */           AsupLRUCacheImpl.this.stats.eagerEvictionRun();
/* 402 */           StringBuffer buffer = new StringBuffer();
/* 403 */           buffer.append("The cache is reduced form the initial size:");
/* 404 */           buffer.append(startSize);
/* 405 */           buffer.append(", with number of entries: ");
/* 406 */           buffer.append(startNumberOfCacheEntries);
/* 407 */           buffer.append(" to the new size:");
/* 408 */           buffer.append(currentSize);
/* 409 */           buffer.append(", with number of entries: ");
/* 410 */           buffer.append(currentNumberOfCacheEntries);
/* 411 */           buffer.append(".Number of Eagerly evicted entries form the cache:");
/* 412 */           buffer.append(removedEntriesCount);
/* 413 */           AsupLRUCacheImpl.LOGGER.info("-9999", buffer.toString());
/*     */         }
/*     */         
/*     */       }
/*     */       catch (Throwable t)
/*     */       {
/* 419 */         AsupLRUCacheImpl.LOGGER.warn("-9999", "Exception occured during the execution of cache eviction.", t.getMessage());
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public long getNumberOfEntriesToBeRemoved(long initialSize, long entriesCount)
/*     */     {
/* 429 */       long toBeRemovedEntries = 0L;
/* 430 */       switch (AsupLRUCacheImpl.1.$SwitchMap$com$netapp$asup$cache$LRU$WeightType[AsupLRUCacheImpl.this.weightType.ordinal()]) {
/*     */       case 1: 
/* 432 */         if (entriesCount >= this.newShrinkedValue) {
/* 433 */           toBeRemovedEntries = entriesCount - this.newShrinkedValue + 1L;
/*     */         }
/*     */         
/*     */         break;
/*     */       case 2: 
/* 438 */         long averageInduvidualsize = initialSize / entriesCount;
/*     */         
/*     */ 
/* 441 */         long approxNumberOfEntries = this.newShrinkedValue / averageInduvidualsize;
/*     */         
/*     */ 
/*     */ 
/* 445 */         toBeRemovedEntries = entriesCount - approxNumberOfEntries + 1L;
/*     */       }
/*     */       
/*     */       
/* 449 */       return toBeRemovedEntries;
/*     */     }
/*     */   }
/*     */   
/*     */   public static class ThresholdUnit {
/* 454 */     private int value = 0;
/* 455 */     private double percentage = 0.0D;
/*     */     
/*     */     public ThresholdUnit(int value, double percentage) {
/* 458 */       this.value = value;
/* 459 */       this.percentage = percentage;
/*     */     }
/*     */     
/*     */     public int unitsValue() {
/* 463 */       int unitValue = (int)(this.value * this.percentage);
/* 464 */       return unitValue;
/*     */     }
/*     */   }
/*     */   
/*     */   static class StatisticsThreadFactory implements ThreadFactory
/*     */   {
/*     */     public Thread newThread(Runnable r) {
/* 471 */       return new Thread(r, "AsupCache.StatisticsThread");
/*     */     }
/*     */   }
/*     */   
/*     */   static class EvictionThreadFactory implements ThreadFactory {
/*     */     public Thread newThread(Runnable r) {
/* 477 */       return new Thread(r, "AsupCache.ThresholdEviction");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   class AsupCacheEvictionListener<String, Object>
/*     */     implements EvictionListener<K, V>
/*     */   {
/*     */     AsupCacheEvictionListener() {}
/*     */     
/*     */     public void onEviction(K key, V value)
/*     */     {
/* 489 */       AsupLRUCacheImpl.this.stats.evict();
/* 490 */       AsupLRUCacheImpl.this.logEviction(key, value, false);
/*     */     }
/*     */   }
/*     */   
/*     */   class DefaultEvictionListener<String, Object> implements EvictionListener<K, V>
/*     */   {
/*     */     DefaultEvictionListener() {}
/*     */     
/*     */     public void onEviction(K key, V value)
/*     */     {
/* 500 */       for (EvictionListener<K, V> listener : AsupLRUCacheImpl.this.registeredListeners) {
/* 501 */         listener.onEviction(key, value);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public void addEvictionListener(EvictionListener<K, V> listener)
/*     */   {
/* 509 */     this.registeredListeners.add(listener);
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\cache\LRU\AsupLRUCacheImpl.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */