/*     */ package com.netapp.asup.data.function.filter;
/*     */ 
/*     */ import com.netapp.asup.data.function.FunctionAttribute;
/*     */ import com.netapp.asup.data.function.FunctionUtil;
/*     */ import com.netapp.asup.data.function.InputType;
/*     */ import com.netapp.asup.data.function.RegisteredFunction;
/*     */ import com.netapp.asup.data.function.TypeConverter;
/*     */ import java.util.Map;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.Schema.Field;
/*     */ import org.apache.avro.generic.GenericData.Array;
/*     */ import org.apache.avro.generic.GenericData.Record;
/*     */ import org.apache.avro.generic.GenericRecord;
/*     */ import org.apache.avro.util.Utf8;
/*     */ import org.codehaus.commons.compiler.CompileException;
/*     */ import org.codehaus.janino.ExpressionEvaluator;
/*     */ 
/*     */ 
/*     */ public class FilterField
/*     */ {
/*     */   private static final String CUSTOM_FUNCTION = "customFunction";
/*     */   private static final String FILTER_TYPE = "filterType";
/*     */   private static final String VALUES = "values";
/*     */   private static final String ATTRIBUTES = "attributes";
/*     */   private static final String ATTR_NAME = "attributeName";
/*     */   private static final String EXPRESSION = "expression";
/*     */   private static final String ATTR_TYPES = "attributeTypes";
/*     */   private static final String OPTIONS = "options";
/*     */   private final String attrName;
/*     */   private final String[] values;
/*     */   private final String[] attributes;
/*     */   private final Class<?>[] attributeTypes;
/*     */   private final String expression;
/*     */   private final RegisteredFunction<?, ?> customFunction;
/*     */   private final Map<String, String> options;
/*     */   private final InputType filterType;
/*     */   
/*     */   FilterField(String attrName, String[] values)
/*     */   {
/*  40 */     this.filterType = InputType.FIELD;
/*  41 */     this.attrName = attrName;
/*  42 */     this.values = values;
/*  43 */     this.expression = null;
/*  44 */     this.attributes = new String[0];
/*  45 */     this.attributeTypes = new Class[0];
/*  46 */     this.customFunction = null;
/*  47 */     this.options = null;
/*     */   }
/*     */   
/*     */   FilterField(String attrName, String expression, FunctionAttribute... expressionAttributes) {
/*  51 */     this.filterType = InputType.EXPRESSION;
/*  52 */     this.attrName = attrName;
/*  53 */     this.expression = expression;
/*  54 */     this.values = new String[0];
/*  55 */     this.customFunction = null;
/*  56 */     int size = expressionAttributes.length;
/*  57 */     this.attributes = new String[size];
/*  58 */     this.attributeTypes = new Class[size];
/*  59 */     this.options = null;
/*  60 */     compileExpression(expressionAttributes);
/*     */   }
/*     */   
/*     */   FilterField(RegisteredFunction<?, ?> function, Map<String, String> options) {
/*  64 */     this.filterType = InputType.FUNCTION;
/*  65 */     this.attrName = null;
/*  66 */     this.values = new String[0];
/*  67 */     this.expression = null;
/*  68 */     this.attributes = new String[0];
/*  69 */     this.attributeTypes = new Class[0];
/*  70 */     this.customFunction = function;
/*  71 */     this.options = options;
/*     */   }
/*     */   
/*     */   public GenericRecord getObject(Schema filterInputSchema) {
/*  75 */     GenericRecord filterInputData = new GenericData.Record(filterInputSchema);
/*     */     
/*     */ 
/*  78 */     Schema attrSchema = filterInputSchema.getField("attributes").schema();
/*  79 */     Schema valuesSchema = filterInputSchema.getField("values").schema();
/*  80 */     Schema attrTypeSchema = filterInputSchema.getField("attributeTypes").schema();
/*     */     
/*  82 */     filterInputData.put("filterType", new Utf8(this.filterType.getName()));
/*  83 */     filterInputData.put("values", loadArray(this.values, valuesSchema));
/*  84 */     filterInputData.put("attributeTypes", loadArray(this.attributeTypes, attrTypeSchema));
/*  85 */     filterInputData.put("attributes", loadArray(this.attributes, attrSchema));
/*     */     
/*  87 */     switch (this.filterType) {
/*     */     case FIELD: 
/*  89 */       filterInputData.put("attributeName", new Utf8(this.attrName));
/*  90 */       break;
/*     */     case EXPRESSION: 
/*  92 */       filterInputData.put("attributeName", new Utf8(this.attrName));
/*  93 */       filterInputData.put("expression", new Utf8(this.expression));
/*  94 */       break;
/*     */     case FUNCTION: 
/*  96 */       filterInputData.put("customFunction", new Utf8(this.customFunction.getFunctionName()));
/*  97 */       filterInputData.put("options", FunctionUtil.convertStringToUtf8(this.options));
/*     */     }
/*     */     
/*     */     
/* 101 */     return filterInputData;
/*     */   }
/*     */   
/*     */ 
/*     */   private GenericData.Array<Utf8> loadArray(Class<?>[] attributeTypes, Schema attributeTypeSchema)
/*     */   {
/* 107 */     if (attributeTypes == null) {
/* 108 */       return new GenericData.Array(0, attributeTypeSchema);
/*     */     }
/*     */     
/* 111 */     GenericData.Array<Utf8> arr = new GenericData.Array(attributeTypes.length, attributeTypeSchema);
/*     */     
/*     */ 
/* 114 */     for (Class<?> attributeType : attributeTypes) {
/* 115 */       arr.add(new Utf8(attributeType.getName()));
/*     */     }
/*     */     
/* 118 */     return arr;
/*     */   }
/*     */   
/*     */ 
/*     */   private GenericData.Array<Utf8> loadArray(String[] attributes, Schema arraySchema)
/*     */   {
/* 124 */     if (attributes == null) {
/* 125 */       return new GenericData.Array(0, arraySchema);
/*     */     }
/* 127 */     GenericData.Array<Utf8> arr = new GenericData.Array(attributes.length, arraySchema);
/*     */     
/* 129 */     for (String attribute : attributes) {
/* 130 */       arr.add(new Utf8(attribute));
/*     */     }
/* 132 */     return arr;
/*     */   }
/*     */   
/*     */ 
/*     */   private void compileExpression(FunctionAttribute... expressionAttributes)
/*     */   {
/* 138 */     int index = 0;
/* 139 */     for (FunctionAttribute attr : expressionAttributes) {
/* 140 */       Class<?> sortAttributeType = attr.getAttrType();
/* 141 */       validateType(sortAttributeType);
/* 142 */       this.attributes[index] = attr.getAttrName();
/* 143 */       this.attributeTypes[index] = sortAttributeType;
/* 144 */       index++;
/*     */     }
/*     */     try
/*     */     {
/* 148 */       new ExpressionEvaluator(this.expression, Object.class, this.attributes, this.attributeTypes);
/*     */     }
/*     */     catch (CompileException e) {
/* 151 */       throw new IllegalArgumentException("Invalid expression " + this.expression + ", caused by :" + e.getCause(), e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   private void validateType(Class<?> sortAttributeType)
/*     */   {
/* 158 */     if (null == sortAttributeType) {
/* 159 */       throw new IllegalArgumentException("Null attribute type is not supported");
/*     */     }
/*     */     
/*     */ 
/* 163 */     if (!TypeConverter.getConvertorclassmap().containsKey(sortAttributeType))
/*     */     {
/* 165 */       throw new IllegalArgumentException("Un supported attributeType.Type:" + sortAttributeType.getName());
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\data\function\filter\FilterField.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */