/*    */ package com.netapp.asup.data.pojo;
/*    */ 
/*    */ import com.netapp.asup.common.AsupLogger;
/*    */ import com.netapp.asup.config.ARMAConfiguration;
/*    */ import com.netapp.asup.exception.ARMAParserException;
/*    */ import com.netapp.asup.pojo.ARMAPojoBuilder;
/*    */ import java.io.InputStream;
/*    */ import java.util.HashMap;
/*    */ import java.util.Map;
/*    */ import javax.xml.parsers.SAXParser;
/*    */ import javax.xml.parsers.SAXParserFactory;
/*    */ import org.apache.avro.generic.GenericRecord;
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ public class ARMAPOJORegistryImpl
/*    */   implements ARMAPOJORegistry
/*    */ {
/* 21 */   private static final AsupLogger LOG = AsupLogger.getLogger(ARMAPOJORegistryImpl.class);
/*    */   
/*    */   private static final String POJO_REGISTRY = "POJORegistry";
/*    */   
/* 25 */   private Map<String, Map<String, ARMAPojoBuilder>> packageToBuilderMap = new HashMap();
/*    */   
/*    */   public ARMAPOJORegistryImpl(ARMAConfiguration config) {
/* 28 */     String pojoArtifactName = config.getString("arma.pojo.artifact.name");
/* 29 */     if (pojoArtifactName != null) {
/* 30 */       String[] pojoArtifactNames = pojoArtifactName.split(",");
/*    */       
/* 32 */       for (String artifactName : pojoArtifactNames) {
/* 33 */         String pojoRegistryName = "POJORegistry_" + artifactName.trim() + ".xml";
/* 34 */         InputStream is = ClassLoader.getSystemClassLoader().getResourceAsStream(pojoRegistryName);
/*    */         
/* 36 */         if (is != null) {
/* 37 */           registerPOJO(is, pojoRegistryName);
/* 38 */           LOG.info(null, "POJO registry " + pojoRegistryName + " parsed success..");
/* 39 */         } else { LOG.warn(null, "POJO registry with name : " + pojoRegistryName + " not found in the classpath ");
/*    */         }
/*    */       }
/* 42 */     } else { LOG.info(null, "no configuration property for arma.pojo.artifact.name configured, POJO functionality may not work..");
/*    */     }
/*    */   }
/*    */   
/*    */   public void registerPOJO(InputStream is, String pojoRegistryName)
/*    */   {
/*    */     try {
/* 49 */       SAXParser parser = getParser();
/* 50 */       POJORegistryHandler handler = new POJORegistryHandler();
/* 51 */       parser.parse(is, handler);
/* 52 */       Map<String, Map<String, ARMAPojoBuilder>> pojoMap = handler.getPackageToBuilder();
/* 53 */       LOG.debug(null, "pojo map while parsing " + pojoRegistryName + " " + pojoMap);
/* 54 */       this.packageToBuilderMap.putAll(pojoMap);
/*    */     }
/*    */     catch (Exception ex) {
/* 57 */       LOG.warn(null, "error encountered while parsing POJO registry : " + ex.getMessage());
/*    */     }
/*    */     
/* 60 */     LOG.debug(null, "POJO registry parse success : " + this.packageToBuilderMap.toString());
/*    */   }
/*    */   
/*    */ 
/*    */ 
/*    */   public ARMAPojoBuilder getBuilder(String packageName, String objectName)
/*    */   {
/* 67 */     if (this.packageToBuilderMap.containsKey(packageName))
/*    */     {
/* 69 */       Map<String, ARMAPojoBuilder> builderMap = (Map)this.packageToBuilderMap.get(packageName);
/*    */       
/*    */ 
/* 72 */       return (ARMAPojoBuilder)builderMap.get(objectName);
/*    */     }
/*    */     
/* 75 */     return null;
/*    */   }
/*    */   
/*    */ 
/*    */   public ARMAPojoBuilder getPOJO(ARMAPojoBuilder builder, GenericRecord input, Map<String, String> derivedAttributes, String systemKey, String instanceKey)
/*    */   {
/* 81 */     return builder.build(input, derivedAttributes, systemKey, instanceKey);
/*    */   }
/*    */   
/*    */   private SAXParser getParser()
/*    */     throws ARMAParserException
/*    */   {
/* 87 */     SAXParserFactory factory = null;
/*    */     try {
/* 89 */       factory = SAXParserFactory.newInstance();
/*    */       
/* 91 */       factory.setValidating(true);
/*    */       
/* 93 */       factory.setFeature("http://apache.org/xml/features/validation/schema", true);
/* 94 */       return factory.newSAXParser();
/*    */     } catch (Exception ex) {
/* 96 */       String errMsg = "Problem initializing the parser factory";
/* 97 */       throw new ARMAParserException(LOG, "-9999", errMsg, ex);
/*    */     }
/*    */   }
/*    */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\data\pojo\ARMAPOJORegistryImpl.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */