/*     */ package net.rubyeye.xmemcached.impl;
/*     */ 
/*     */ import com.google.code.yanf4j.core.Session;
/*     */ import java.net.InetAddress;
/*     */ import java.net.InetSocketAddress;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Collection;
/*     */ import java.util.List;
/*     */ import java.util.Random;
/*     */ import java.util.SortedMap;
/*     */ import java.util.TreeMap;
/*     */ import net.rubyeye.xmemcached.HashAlgorithm;
/*     */ import net.rubyeye.xmemcached.networking.MemcachedSession;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class KetamaMemcachedSessionLocator
/*     */   extends AbstractMemcachedSessionLocator
/*     */ {
/*     */   static final int NUM_REPS = 160;
/*  53 */   private volatile transient TreeMap<Long, List<Session>> ketamaSessions = new TreeMap();
/*     */   private final HashAlgorithm hashAlg;
/*     */   private volatile int maxTries;
/*  56 */   private final Random random = new Random();
/*     */   
/*     */   static final int DEFAULT_PORT = 11211;
/*     */   
/*     */   private final boolean cwNginxUpstreamConsistent;
/*     */   
/*     */ 
/*     */   public KetamaMemcachedSessionLocator()
/*     */   {
/*  65 */     this.hashAlg = HashAlgorithm.KETAMA_HASH;
/*  66 */     this.cwNginxUpstreamConsistent = false;
/*     */   }
/*     */   
/*     */   public KetamaMemcachedSessionLocator(boolean cwNginxUpstreamConsistent) {
/*  70 */     this.hashAlg = HashAlgorithm.KETAMA_HASH;
/*  71 */     this.cwNginxUpstreamConsistent = cwNginxUpstreamConsistent;
/*     */   }
/*     */   
/*     */   public KetamaMemcachedSessionLocator(HashAlgorithm alg) {
/*  75 */     this.hashAlg = alg;
/*  76 */     this.cwNginxUpstreamConsistent = false;
/*     */   }
/*     */   
/*     */   public KetamaMemcachedSessionLocator(HashAlgorithm alg, boolean cwNginxUpstreamConsistent)
/*     */   {
/*  81 */     this.hashAlg = alg;
/*  82 */     this.cwNginxUpstreamConsistent = cwNginxUpstreamConsistent;
/*     */   }
/*     */   
/*     */   public KetamaMemcachedSessionLocator(List<Session> list, HashAlgorithm alg)
/*     */   {
/*  87 */     this.hashAlg = alg;
/*  88 */     this.cwNginxUpstreamConsistent = false;
/*  89 */     buildMap(list, alg);
/*     */   }
/*     */   
/*     */   private final void buildMap(Collection<Session> list, HashAlgorithm alg) {
/*  93 */     TreeMap<Long, List<Session>> sessionMap = new TreeMap();
/*     */     
/*     */ 
/*  96 */     for (Session session : list) { String sockStr;
/*  97 */       if (this.cwNginxUpstreamConsistent) {
/*  98 */         InetSocketAddress serverAddress = session.getRemoteSocketAddress();
/*     */         
/* 100 */         String sockStr = serverAddress.getAddress().getHostAddress();
/* 101 */         if (serverAddress.getPort() != 11211) {
/* 102 */           sockStr = sockStr + ":" + serverAddress.getPort();
/*     */         }
/*     */       } else {
/* 105 */         sockStr = String.valueOf(session.getRemoteSocketAddress());
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 110 */       int numReps = 160;
/* 111 */       if ((session instanceof MemcachedTCPSession)) {
/* 112 */         numReps *= ((MemcachedSession)session).getWeight();
/*     */       }
/* 114 */       if (alg == HashAlgorithm.KETAMA_HASH) {
/* 115 */         for (int i = 0; i < numReps / 4; i++) {
/* 116 */           byte[] digest = HashAlgorithm.computeMd5(sockStr + "-" + i);
/* 117 */           for (int h = 0; h < 4; h++) {
/* 118 */             long k = (digest[(3 + h * 4)] & 0xFF) << 24 | (digest[(2 + h * 4)] & 0xFF) << 16 | (digest[(1 + h * 4)] & 0xFF) << 8 | digest[(h * 4)] & 0xFF;
/*     */             
/*     */ 
/*     */ 
/* 122 */             getSessionList(sessionMap, k).add(session);
/*     */           }
/*     */           
/*     */         }
/*     */       } else {
/* 127 */         for (int i = 0; i < numReps; i++) {
/* 128 */           long key = alg.hash(sockStr + "-" + i);
/* 129 */           getSessionList(sessionMap, key).add(session);
/*     */         }
/*     */       }
/*     */     }
/* 133 */     this.ketamaSessions = sessionMap;
/* 134 */     this.maxTries = list.size();
/*     */   }
/*     */   
/*     */   private List<Session> getSessionList(TreeMap<Long, List<Session>> sessionMap, long k)
/*     */   {
/* 139 */     List<Session> sessionList = (List)sessionMap.get(Long.valueOf(k));
/* 140 */     if (sessionList == null) {
/* 141 */       sessionList = new ArrayList();
/* 142 */       sessionMap.put(Long.valueOf(k), sessionList);
/*     */     }
/* 144 */     return sessionList;
/*     */   }
/*     */   
/*     */   public final Session getSessionByKey(String key) {
/* 148 */     if ((this.ketamaSessions == null) || (this.ketamaSessions.size() == 0)) {
/* 149 */       return null;
/*     */     }
/* 151 */     long hash = this.hashAlg.hash(key);
/* 152 */     Session rv = getSessionByHash(hash);
/* 153 */     int tries = 0;
/*     */     
/* 155 */     while ((!this.failureMode) && ((rv == null) || (rv.isClosed())) && (tries++ < this.maxTries)) {
/* 156 */       hash = nextHash(hash, key, tries);
/* 157 */       rv = getSessionByHash(hash);
/*     */     }
/* 159 */     return rv;
/*     */   }
/*     */   
/*     */   public final Session getSessionByHash(long hash) {
/* 163 */     TreeMap<Long, List<Session>> sessionMap = this.ketamaSessions;
/* 164 */     if (sessionMap.size() == 0) {
/* 165 */       return null;
/*     */     }
/* 167 */     Long resultHash = Long.valueOf(hash);
/* 168 */     if (!sessionMap.containsKey(Long.valueOf(hash)))
/*     */     {
/*     */ 
/* 171 */       SortedMap<Long, List<Session>> tailMap = sessionMap.tailMap(Long.valueOf(hash));
/* 172 */       if (tailMap.isEmpty()) {
/* 173 */         resultHash = (Long)sessionMap.firstKey();
/*     */       } else {
/* 175 */         resultHash = (Long)tailMap.firstKey();
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 185 */     List<Session> sessionList = (List)sessionMap.get(resultHash);
/* 186 */     if ((sessionList == null) || (sessionList.size() == 0)) {
/* 187 */       return null;
/*     */     }
/* 189 */     int size = sessionList.size();
/* 190 */     return (Session)sessionList.get(this.random.nextInt(size));
/*     */   }
/*     */   
/*     */   public final long nextHash(long hashVal, String key, int tries) {
/* 194 */     long tmpKey = this.hashAlg.hash(tries + key);
/* 195 */     hashVal += (int)(tmpKey ^ tmpKey >>> 32);
/* 196 */     hashVal &= 0xFFFFFFFF;
/* 197 */     return hashVal;
/*     */   }
/*     */   
/*     */   public final void updateSessions(Collection<Session> list) {
/* 201 */     buildMap(list, this.hashAlg);
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\net\rubyeye\xmemcached\impl\KetamaMemcachedSessionLocator.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */