/*     */ package net.rubyeye.xmemcached.impl;
/*     */ 
/*     */ import com.google.code.yanf4j.core.WriteMessage;
/*     */ import com.google.code.yanf4j.core.impl.FutureImpl;
/*     */ import com.google.code.yanf4j.nio.NioSessionConfig;
/*     */ import com.google.code.yanf4j.nio.impl.NioTCPSession;
/*     */ import com.google.code.yanf4j.util.LinkedTransferQueue;
/*     */ import com.google.code.yanf4j.util.SystemUtils;
/*     */ import java.net.InetSocketAddress;
/*     */ import java.net.Socket;
/*     */ import java.net.SocketAddress;
/*     */ import java.net.SocketException;
/*     */ import java.nio.channels.SocketChannel;
/*     */ import java.util.concurrent.BlockingQueue;
/*     */ import java.util.concurrent.CountDownLatch;
/*     */ import java.util.concurrent.Future;
/*     */ import java.util.concurrent.atomic.AtomicReference;
/*     */ import net.rubyeye.xmemcached.CommandFactory;
/*     */ import net.rubyeye.xmemcached.MemcachedOptimizer;
/*     */ import net.rubyeye.xmemcached.buffer.BufferAllocator;
/*     */ import net.rubyeye.xmemcached.command.Command;
/*     */ import net.rubyeye.xmemcached.command.OperationStatus;
/*     */ import net.rubyeye.xmemcached.exception.MemcachedException;
/*     */ import net.rubyeye.xmemcached.networking.MemcachedSession;
/*     */ import net.rubyeye.xmemcached.utils.InetSocketAddressWrapper;
/*     */ import org.slf4j.Logger;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class MemcachedTCPSession
/*     */   extends NioTCPSession
/*     */   implements MemcachedSession
/*     */ {
/*     */   protected BlockingQueue<Command> commandAlreadySent;
/*  51 */   private final AtomicReference<Command> currentCommand = new AtomicReference();
/*     */   
/*     */   private SocketAddress remoteSocketAddress;
/*     */   
/*     */   private int sendBufferSize;
/*     */   
/*     */   private final MemcachedOptimizer optimiezer;
/*     */   
/*     */   private volatile boolean allowReconnect;
/*     */   private volatile boolean authFailed;
/*     */   private final CommandFactory commandFactory;
/*     */   private InetSocketAddressWrapper inetSocketAddressWrapper;
/*     */   private BufferAllocator bufferAllocator;
/*     */   
/*     */   public MemcachedTCPSession(NioSessionConfig sessionConfig, int readRecvBufferSize, MemcachedOptimizer optimiezer, int readThreadCount, CommandFactory commandFactory)
/*     */   {
/*  67 */     super(sessionConfig, readRecvBufferSize);
/*  68 */     this.optimiezer = optimiezer;
/*  69 */     if (this.selectableChannel != null) {
/*  70 */       this.remoteSocketAddress = ((SocketChannel)this.selectableChannel).socket().getRemoteSocketAddress();
/*     */       
/*  72 */       this.allowReconnect = true;
/*     */       try {
/*  74 */         this.sendBufferSize = ((SocketChannel)this.selectableChannel).socket().getSendBufferSize();
/*     */       }
/*     */       catch (SocketException e) {
/*  77 */         this.sendBufferSize = 8192;
/*     */       }
/*     */     }
/*  80 */     this.commandAlreadySent = new LinkedTransferQueue();
/*  81 */     this.commandFactory = commandFactory;
/*     */   }
/*     */   
/*     */   public InetSocketAddressWrapper getInetSocketAddressWrapper() {
/*  85 */     return this.inetSocketAddressWrapper;
/*     */   }
/*     */   
/*     */   public int getOrder() {
/*  89 */     return getInetSocketAddressWrapper().getOrder();
/*     */   }
/*     */   
/*     */   public int getWeight() {
/*  93 */     return getInetSocketAddressWrapper().getWeight();
/*     */   }
/*     */   
/*     */   public void setInetSocketAddressWrapper(InetSocketAddressWrapper inetSocketAddressWrapper)
/*     */   {
/*  98 */     this.inetSocketAddressWrapper = inetSocketAddressWrapper;
/*     */   }
/*     */   
/*     */   public String toString()
/*     */   {
/* 103 */     return SystemUtils.getRawAddress(getRemoteSocketAddress()) + ":" + getRemoteSocketAddress().getPort();
/*     */   }
/*     */   
/*     */   public void destroy()
/*     */   {
/* 108 */     Command command = (Command)this.currentCommand.get();
/* 109 */     if (command != null) {
/* 110 */       command.setException(new MemcachedException("Session has been closed"));
/*     */       
/* 112 */       command.getLatch().countDown();
/*     */     }
/* 114 */     while ((command = (Command)this.commandAlreadySent.poll()) != null) {
/* 115 */       command.setException(new MemcachedException("Session has been closed"));
/*     */       
/* 117 */       if (command.getLatch() != null) {
/* 118 */         command.getLatch().countDown();
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public InetSocketAddress getRemoteSocketAddress()
/*     */   {
/* 126 */     InetSocketAddress result = super.getRemoteSocketAddress();
/* 127 */     if ((result == null) && (this.remoteSocketAddress != null)) {
/* 128 */       result = (InetSocketAddress)this.remoteSocketAddress;
/*     */     }
/* 130 */     return result;
/*     */   }
/*     */   
/*     */   protected WriteMessage preprocessWriteMessage(WriteMessage writeMessage)
/*     */   {
/* 135 */     Command currentCommand = (Command)writeMessage;
/*     */     
/* 137 */     if (currentCommand.getIoBuffer() == null) {
/* 138 */       currentCommand.encode();
/*     */     }
/* 140 */     if (currentCommand.getStatus() == OperationStatus.SENDING)
/*     */     {
/*     */ 
/*     */ 
/* 144 */       currentCommand = this.optimiezer.optimize(currentCommand, this.writeQueue, this.commandAlreadySent, this.sendBufferSize);
/*     */     }
/*     */     
/*     */ 
/* 148 */     currentCommand.setStatus(OperationStatus.WRITING);
/* 149 */     return currentCommand;
/*     */   }
/*     */   
/*     */   public boolean isAuthFailed() {
/* 153 */     return this.authFailed;
/*     */   }
/*     */   
/*     */   public void setAuthFailed(boolean authFailed) {
/* 157 */     this.authFailed = authFailed;
/*     */   }
/*     */   
/*     */ 
/*     */   public final BufferAllocator getBufferAllocator()
/*     */   {
/* 163 */     return this.bufferAllocator;
/*     */   }
/*     */   
/*     */   public final void setBufferAllocator(BufferAllocator bufferAllocator) {
/* 167 */     this.bufferAllocator = bufferAllocator;
/*     */   }
/*     */   
/*     */ 
/*     */   protected final WriteMessage wrapMessage(Object msg, Future<Boolean> writeFuture)
/*     */   {
/* 173 */     ((Command)msg).encode();
/* 174 */     ((Command)msg).setWriteFuture((FutureImpl)writeFuture);
/* 175 */     if (log.isDebugEnabled()) {
/* 176 */       log.debug("After encoding" + ((Command)msg).toString());
/*     */     }
/* 178 */     return (WriteMessage)msg;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private Command takeExecutingCommand()
/*     */   {
/*     */     try
/*     */     {
/* 188 */       return (Command)this.commandAlreadySent.take();
/*     */     } catch (InterruptedException e) {
/* 190 */       Thread.currentThread().interrupt();
/*     */     }
/* 192 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public boolean isAllowReconnect()
/*     */   {
/* 201 */     return this.allowReconnect;
/*     */   }
/*     */   
/*     */   public void setAllowReconnect(boolean reconnected) {
/* 205 */     this.allowReconnect = reconnected;
/*     */   }
/*     */   
/*     */   public void addCommand(Command command) {
/* 209 */     this.commandAlreadySent.add(command);
/*     */   }
/*     */   
/*     */   public void setCurrentCommand(Command cmd) {
/* 213 */     this.currentCommand.set(cmd);
/*     */   }
/*     */   
/*     */   public Command getCurrentCommand() {
/* 217 */     return (Command)this.currentCommand.get();
/*     */   }
/*     */   
/*     */   public void takeCurrentCommand() {
/* 221 */     setCurrentCommand(takeExecutingCommand());
/*     */   }
/*     */   
/*     */   public void quit() {
/* 225 */     write(this.commandFactory.createQuitCommand());
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\net\rubyeye\xmemcached\impl\MemcachedTCPSession.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */