package prefuse.data.column;

import java.util.Arrays;
import prefuse.data.DataReadOnlyException;
import prefuse.data.DataTypeException;

public class IntColumn
  extends AbstractColumn
{
  private int[] m_values;
  private int m_size;
  
  public IntColumn()
  {
    this(0, 10, -1);
  }
  
  public IntColumn(int paramInt)
  {
    this(paramInt, paramInt, -1);
  }
  
  public IntColumn(int paramInt1, int paramInt2, int paramInt3)
  {
    super(Integer.TYPE, new Integer(paramInt3));
    if (paramInt2 < paramInt1) {
      throw new IllegalArgumentException("Capacity value can not be less than the row count.");
    }
    this.m_values = new int[paramInt2];
    Arrays.fill(this.m_values, paramInt3);
    this.m_size = paramInt1;
  }
  
  public int getRowCount()
  {
    return this.m_size;
  }
  
  public void setMaximumRow(int paramInt)
  {
    if (paramInt > this.m_values.length)
    {
      int i = Math.max(3 * this.m_values.length / 2 + 1, paramInt);
      int[] arrayOfInt = new int[i];
      System.arraycopy(this.m_values, 0, arrayOfInt, 0, this.m_size);
      Arrays.fill(arrayOfInt, this.m_size, i, ((Integer)this.m_defaultValue).intValue());
      this.m_values = arrayOfInt;
    }
    this.m_size = paramInt;
  }
  
  public Object get(int paramInt)
  {
    return new Integer(getInt(paramInt));
  }
  
  public void set(Object paramObject, int paramInt)
    throws DataTypeException
  {
    if (this.m_readOnly) {
      throw new DataReadOnlyException();
    }
    if (paramObject != null)
    {
      if ((paramObject instanceof Number)) {
        setInt(((Number)paramObject).intValue(), paramInt);
      } else if ((paramObject instanceof String)) {
        setString((String)paramObject, paramInt);
      } else {
        throw new DataTypeException(paramObject.getClass());
      }
    }
    else {
      throw new DataTypeException("Column does not accept null values");
    }
  }
  
  public int getInt(int paramInt)
    throws DataTypeException
  {
    if ((paramInt < 0) || (paramInt > this.m_size)) {
      throw new IllegalArgumentException("Row index out of bounds: " + paramInt);
    }
    return this.m_values[paramInt];
  }
  
  public void setInt(int paramInt1, int paramInt2)
    throws DataTypeException
  {
    if (this.m_readOnly) {
      throw new DataReadOnlyException();
    }
    if ((paramInt2 < 0) || (paramInt2 >= this.m_size)) {
      throw new IllegalArgumentException("Row index out of bounds: " + paramInt2);
    }
    int i = this.m_values[paramInt2];
    if (i == paramInt1) {
      return;
    }
    this.m_values[paramInt2] = paramInt1;
    fireColumnEvent(paramInt2, i);
  }
  
  public String getString(int paramInt)
    throws DataTypeException
  {
    return String.valueOf(getInt(paramInt));
  }
  
  public void setString(String paramString, int paramInt)
    throws DataTypeException
  {
    setInt(Integer.parseInt(paramString), paramInt);
  }
  
  public long getLong(int paramInt)
    throws DataTypeException
  {
    return getInt(paramInt);
  }
  
  public float getFloat(int paramInt)
    throws DataTypeException
  {
    return getInt(paramInt);
  }
  
  public double getDouble(int paramInt)
    throws DataTypeException
  {
    return getInt(paramInt);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\column\IntColumn.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */