package prefuse.data.util;

import prefuse.data.Table;
import prefuse.util.collections.IntIntSortedMap;
import prefuse.util.collections.IntIntTreeMap;
import prefuse.util.collections.IntIterator;

public class RowManager
{
  protected Table m_table;
  private IntIntSortedMap m_openrows;
  private int m_firstid = 0;
  private int m_curid = -1;
  
  public RowManager(Table paramTable)
  {
    this.m_table = paramTable;
  }
  
  public Table getTable()
  {
    return this.m_table;
  }
  
  public int getMinimumRow()
  {
    return this.m_firstid;
  }
  
  public int getMaximumRow()
  {
    return this.m_curid;
  }
  
  public int getRowCount()
  {
    return 1 + this.m_curid - this.m_firstid - (this.m_openrows == null ? 0 : this.m_openrows.size());
  }
  
  public boolean isValidRow(int paramInt)
  {
    return (paramInt >= this.m_firstid) && (paramInt <= this.m_curid) && ((this.m_openrows == null) || (!this.m_openrows.containsKey(paramInt)));
  }
  
  public void clear()
  {
    this.m_openrows = null;
    this.m_firstid = 0;
    this.m_curid = -1;
  }
  
  public int addRow()
  {
    int i;
    if ((this.m_openrows == null) || (this.m_openrows.isEmpty()))
    {
      i = --this.m_firstid;
    }
    else
    {
      int j = this.m_openrows.firstKey();
      i = this.m_openrows.remove(j);
    }
    return i;
  }
  
  public boolean releaseRow(int paramInt)
  {
    if (paramInt < 0) {
      return false;
    }
    if ((this.m_openrows != null) && (this.m_openrows.containsKey(paramInt))) {
      return false;
    }
    if (paramInt == this.m_curid)
    {
      this.m_curid -= 1;
    }
    else if (paramInt == this.m_firstid)
    {
      this.m_firstid += 1;
    }
    else
    {
      if (this.m_openrows == null) {
        this.m_openrows = new IntIntTreeMap(false);
      }
      this.m_openrows.put(paramInt, paramInt);
    }
    return true;
  }
  
  public int getColumnRow(int paramInt1, int paramInt2)
  {
    return isValidRow(paramInt1) ? paramInt1 : -1;
  }
  
  public int getTableRow(int paramInt1, int paramInt2)
  {
    return isValidRow(paramInt1) ? paramInt1 : -1;
  }
  
  public IntIterator columnRows(int paramInt)
  {
    return new ColumnRowIterator(rows(), paramInt);
  }
  
  public IntIterator columnRows(int paramInt, boolean paramBoolean)
  {
    return new ColumnRowIterator(rows(paramBoolean), paramInt);
  }
  
  public IntIterator columnRows(IntIterator paramIntIterator, int paramInt)
  {
    return new ColumnRowIterator(paramIntIterator, paramInt);
  }
  
  public IntIterator rows()
  {
    return new RowIterator(false);
  }
  
  public IntIterator rows(boolean paramBoolean)
  {
    return new RowIterator(paramBoolean);
  }
  
  public class ColumnRowIterator
    extends IntIterator
  {
    private IntIterator rows;
    private int row;
    private int col;
    
    public ColumnRowIterator(IntIterator paramIntIterator, int paramInt)
    {
      this.rows = paramIntIterator;
      this.col = paramInt;
    }
    
    public boolean hasNext()
    {
      return this.rows.hasNext();
    }
    
    public int nextInt()
    {
      this.row = this.rows.nextInt();
      return RowManager.this.getColumnRow(this.row, this.col);
    }
    
    public void remove()
    {
      RowManager.this.m_table.removeRow(this.row);
    }
  }
  
  public class RowIterator
    extends IntIterator
  {
    boolean reverse;
    int last = -1;
    int next;
    
    public RowIterator(boolean paramBoolean)
    {
      this.reverse = paramBoolean;
      this.next = advance(paramBoolean ? RowManager.this.m_curid : RowManager.this.m_firstid);
    }
    
    public boolean hasNext()
    {
      return this.next >= 0;
    }
    
    public int nextInt()
    {
      this.last = this.next;
      this.next = advance(++this.next);
      return this.last;
    }
    
    public void remove()
    {
      RowManager.this.m_table.removeRow(this.last);
    }
    
    private final int advance(int paramInt)
    {
      if (RowManager.this.m_openrows == null) {
        return paramInt;
      }
      if (this.reverse) {
        while ((paramInt >= 0) && (RowManager.this.m_openrows.containsKey(paramInt))) {
          paramInt--;
        }
      }
      while ((paramInt <= RowManager.this.m_curid) && (RowManager.this.m_openrows.containsKey(paramInt))) {
        paramInt++;
      }
      return paramInt;
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\util\RowManager.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */